# -*- coding: utf-8 -*-
"""
Created on Thu Mar 28 15:34:40 2024

@author: SA-Lenovo
"""
import numpy as np
import matplotlib.pyplot as plt

N, M = 256j, 512j
O = np.ones((int(N.imag), int(M.imag), 3))                        # Init image plane origin
O[..., 1], O[..., 0] = np.mgrid[0.5:-0.5:N, 1:-1:M]               # Image plane uvw coordinates
e_ = O/np.linalg.norm(O, axis=2)[:,:,np.newaxis]                  # Normalized ray directon e_

# Triangle 
A = np.array([0   , 2.2 , 5])                                     # Point A
B = np.array([6.7 , -3  , 8])                                     # Point B
C = np.array([-1.5, -0.5, 2])                                     # Point C

AB = B - A                                                        # Oriented segment A to B
AC = C - A                                                        # Oriented segment A to C
n = np.cross(AB, AC)                                              # Normal vector
n_ = n/np.linalg.norm(n)                                          # Normalized normal

# Using the point A to find d
d = - np.dot(n_, A)

# Finding parameter t
vec_dot = np.vectorize(np.dot, signature='(n),(m)->()')           # Vectorize dot product function
t = - (vec_dot(n_, O) + d)/vec_dot(n_, e_)                        # Get t for each pixel

# Finding P
P = O + t[..., np.newaxis]*e_

# Get the resulting vector for each vertex
# following the construction order
Pa = vec_dot(np.cross(B - A, P - A), n_)                          # Resulting vector of A
Pb = vec_dot(np.cross(C - B, P - B), n_)                          # Resulting vector of B
Pc = vec_dot(np.cross(A - C, P - C), n_)                          # Resulting vector of C

output = np.zeros((int(N.imag), int(M.imag), 3))                  # Init output image
cond = np.logical_and(np.logical_and(Pa >= 0, Pb >= 0), Pc >= 0)  # Inside the triangle conditionals
fr = vec_dot(n_, -e_)[..., np.newaxis]                            # Compute the facing ratio
output[cond] = (0.15, 0.35, 0.9)*fr[cond]                         # Shade with color and fr


# Visualization
fig, ax = plt.subplots(figsize=(16, 10))
ax.imshow(output)
plt.show()
