# -*- coding: utf-8 -*-
"""
Created on Fri Mar 29 09:33:16 2024

@author: SA-Lenovo
"""
import matplotlib.pyplot as plt
import numpy as np

N, M = 256j, 512j
O = np.ones((int(N.imag), int(M.imag), 3))                        # Init image plane origin
O[..., 1], O[..., 0] = np.mgrid[0.5:-0.5:N, 1:-1:M]               # Image plane uvw coordinates
e_ = O/np.linalg.norm(O, axis=2)[..., np.newaxis]                 # Normalized ray directon e_

# Triangle 
A = np.array([0 , 1.25 , 3])                                      # Point A
B = np.array([2 , -1.25, 3])                                      # Point B
C = np.array([-2, -1.25, 3])                                      # Point C

AB = B - A                                                        # Oriented segment A to B
AC = C - A                                                        # Oriented segment A to C
n = np.cross(AB, AC)                                              # Normal vector
n_ = n/np.linalg.norm(n)                                          # Normalized normal

# Using the point A to find d
d = - np.dot(n_, A)

# Finding parameter t
vec_dot = np.vectorize(np.dot, signature='(n),(m)->()')           # Vectorize dot product function
t = - (vec_dot(n_, O) + d)/vec_dot(n_, e_)                        # Get t for each pixel

# Finding P
P = O + t[..., np.newaxis]*e_

# Get the resulting vector for each vertex
# following the construction order
Pa = vec_dot(np.cross(B - A, P - A), n_)                          # Resulting vector of A
Pb = vec_dot(np.cross(C - B, P - B), n_)                          # Resulting vector of B
Pc = vec_dot(np.cross(A - C, P - C), n_)                          # Resulting vector of C

cond = np.logical_and(np.logical_and(Pa >= 0, Pb >= 0), Pc >= 0)  # Inside the triangle conditionals

# Calculate barycentric coordinates
Aa = np.cross(B - A, P - A)                                       # Resulting vector of A and P
Aa = np.linalg.norm(Aa, axis=2)                                   # Area of triangle ABP
Ab = np.cross(C - B, P - B)                                       # Resulting vector of B and P
Ab = np.linalg.norm(Ab, axis=2)                                   # Area of triangle BCP
Ac = np.cross(A - C, P - C)                                       # Resulting vector of C and P
Ac = np.linalg.norm(Ac, axis=2)                                   # Area of triangle CAP
At = np.cross(C - A, B - A)                                       # Resulting vector of triangle
At = np.linalg.norm(At)                                           # Area of triangle ABC

# Getting the barycenter weights
alpha = (Ab/At)[..., np.newaxis]
beta = (Ac/At)[..., np.newaxis]
gamma = (Aa/At)[..., np.newaxis]

# Output image
output = np.zeros((int(N.imag), int(M.imag), 3))                  # Init output image
Ca = np.array([1, 0, 0.4])                                        # Color vertex A
Cb = np.array([0.4, 1, 0])                                        # Color vertex B
Cc = np.array([0, 0.4, 1])                                        # Color vertex C
Cd = (alpha*Ca + beta*Cb + gamma*Cc)/(alpha + beta + gamma)       # Interpolated color based on barycentric coordinates

output[cond] = Cd[cond]                                           # Shade with the interpolated colors

# Visualization
fig, ax = plt.subplots(figsize=(16, 10))
ax.imshow(output)
plt.show()
