# -*- coding: utf-8 -*-
"""
Created on Sun Mar 24 10:45:42 2024

@author: SA-Lenovo


3D Intersection

"""

import matplotlib.pyplot as plt
import numpy as np

N, M = 256j, 512j
O = np.ones((int(N.imag), int(M.imag), 3))               # Init image plane origin
O[..., 1], O[..., 0] = np.mgrid[0.5:-0.5:N, 1:-1:M]      # Image plane uvw coordinates
e_ = O/np.linalg.norm(O, axis=2)[:,:,np.newaxis]         # Normalized ray directon e_

# Sphere
Cs = np.array([0, 0, 4])                                 # Center of sphere
r = 1.5                                                  # Radius of sphere

OC_ = Cs - O                                             # Oriented segment from origin to center of the sphere

vec_dot = np.vectorize(np.dot, signature='(n),(m)->()')  # Vectorize dot product function
t = vec_dot(OC_, e_)                                     # Pixelwise dot product
Pe = O + e_*t[:,:,np.newaxis]                            # Point on vector e_ projected from OC_
d = np.linalg.norm(Pe - Cs, axis=2)                      # Distance from the point Pe and the center of the sphere

# Find intersection position
i = (r**2 - d**2)**0.5                                   # 
Ps = O + e_*(t - i)[:,:,np.newaxis]                      # 

# Facing ratio (incidence value)
i_ = i[:,:,np.newaxis]/r

# Calculate the normal vector for each point
n = Ps - Cs                                              # Calculate vector n
n_ = n/np.linalg.norm(n, axis=2)[:,:,np.newaxis]         # Normalize n

##################################
# Simple directional light model #
##################################
Cd = np.array([0.9, 0.15, 0.35])**(1/0.455)              # Sphere diffuse color with gamma compensation

# Key light
l = np.array([-1.5, 1.5, -1])                            # Key light vector
l_ = l/np.linalg.norm(l)                                 # Key light vector normalization
Kd = vec_dot(l_, n_)[:,:,np.newaxis]                     # Calculate light incidence
Kd[Kd < 0] = 0                                           # clamp negative values
diff = Cd*Kd                                             # Writes to diffuse

# Back light
l = np.array([1.5, -1, 1])                               # Back light vector
l_ = l/np.linalg.norm(l)                                 # Back light vector normalization
Kd = vec_dot(l_, n_)[:,:,np.newaxis]                     # Calculate light incidence
Kd[Kd < 0] = 0                                           # clamp negative values
diff += Cd*Kd*0.25                                       # Adds to diffuse

output = np.zeros((int(N.imag), int(M.imag), 3))         # Init output image
output[d < r] = (diff*i_)[d < r]                         # Shades diffuse and fr

output[output < 0] = 0; output[output > 1] = 1           # Clamp values before visualization

# Visualization
fig, ax = plt.subplots(figsize=(16, 10))
ax.imshow(output**(1/2.2))                               # View the resulting image with gamma adjustment (sRGB)
plt.show()
